/******************************************************************************\
* Copyright (C) 2012-2013 Leap Motion, Inc. All rights reserved.               *
* Leap Motion proprietary and confidential. Not for distribution.              *
* Use subject to the terms of the Leap Motion SDK Agreement available at       *
* https://developer.leapmotion.com/sdk_agreement, or another agreement         *
* between Leap Motion and you, your company or other organization.             *
\******************************************************************************/

/*
* Makes this file compilable in Netbeans following this brief guide: https://developer.leapmotion.com/documentation/Languages/Java/Guides/Setup_Java.html#netbeans
* Or you can add the LeapJava.jar in the current project library directory and then move the libraries Leap.dll and LeapJava.dll in System32 windows directory. 
*/

package javaapplication1;
import java.io.IOException;
import java.lang.Math;
import com.leapmotion.leap.*;
import com.leapmotion.leap.Gesture.State;

class SampleListener extends Listener {
    public static boolean play = false; // Music player flag
    public static hue_json HUE = new hue_json(); // starting Philips Hue connection.
    
    public void onInit(Controller controller) {
        System.out.println("Initialized");
    }

    public void onConnect(Controller controller) {
        System.out.println("Connected");
        controller.enableGesture(Gesture.Type.TYPE_SWIPE);
        controller.enableGesture(Gesture.Type.TYPE_CIRCLE);
        controller.enableGesture(Gesture.Type.TYPE_SCREEN_TAP);
        controller.enableGesture(Gesture.Type.TYPE_KEY_TAP);
    }

    public void onDisconnect(Controller controller) {
        //Note: not dispatched when running in a debugger.
        System.out.println("Disconnected");
    }

    public void onExit(Controller controller) {
        System.out.println("Exited");
    }

    public void onFrame(Controller controller) {
        // Get the most recent frame and report some basic information
        Frame frame = controller.frame();
        /*System.out.println("Frame id: " + frame.id()
                         + ", timestamp: " + frame.timestamp()
                         + ", hands: " + frame.hands().count()
                         + ", fingers: " + frame.fingers().count()
                         + ", tools: " + frame.tools().count()
                         + ", gestures " + frame.gestures().count());*/

        if (!frame.hands().isEmpty()) {
            // Get the first hand
            Hand hand = frame.hands().get(0);

            // Check if the hand has any fingers
            FingerList fingers = hand.fingers();
            if (!fingers.isEmpty()) {
                // Calculate the hand's average finger tip position
                Vector avgPos = Vector.zero();
                for (Finger finger : fingers) {
                    avgPos = avgPos.plus(finger.tipPosition());
                }
                avgPos = avgPos.divide(fingers.count());
                /*System.out.println("Hand has " + fingers.count()
                                 + " fingers, average finger tip position: " + avgPos);*/
            }

            // Get the hand's sphere radius and palm position
            /*System.out.println("Hand sphere radius: " + hand.sphereRadius()
                             + " mm, palm position: " + hand.palmPosition());*/

            // Get the hand's normal vector and direction
            Vector normal = hand.palmNormal();
            Vector direction = hand.direction();

            // Calculate the hand's pitch, roll, and yaw angles
            /*System.out.println("Hand pitch: " + Math.toDegrees(direction.pitch()) + " degrees, "
                             + "roll: " + Math.toDegrees(normal.roll()) + " degrees, "
                             + "yaw: " + Math.toDegrees(direction.yaw()) + " degrees");*/
        }

        GestureList gestures = frame.gestures();
        for (int i = 0; i < gestures.count(); i++) {
            Gesture gesture = gestures.get(i);

            switch (gesture.type()) {
                
                case TYPE_CIRCLE:
                    CircleGesture circle = new CircleGesture(gesture);

                    // Calculate clock direction using the angle between circle normal and pointable
                    String clockwiseness;
                    if (circle.pointable().direction().angleTo(circle.normal()) <= Math.PI/4) {
                        // Clockwise if angle is less than 90 degrees
                        clockwiseness = "clockwise";
                    } else {
                        clockwiseness = "counterclockwise";
                    }

                    // Calculate angle swept since last frame
                    double sweptAngle = 0;
                    if (circle.state() != State.STATE_START) {
                        CircleGesture previousUpdate = new CircleGesture(controller.frame(1).gesture(circle.id()));
                        sweptAngle = (circle.progress() - previousUpdate.progress()) * 2 * Math.PI;
                    }

                    System.out.println("Circle id: " + circle.id()
                               + ", " + circle.state()
                               + ", progress: " + circle.progress()
                               + ", radius: " + circle.radius()
                               + ", angle: " + Math.toDegrees(sweptAngle)
                               + ", " + clockwiseness);
                    
                    
                    //processiamo la gesture:
                    process_circle(frame.hands().count(), frame.hands().leftmost().fingers().count(), frame.hands().rightmost().fingers().count(), clockwiseness, circle.progress());
                    
                    break;
                
                case TYPE_SWIPE:
                    SwipeGesture swipe = new SwipeGesture(gesture);
                    System.out.println("Swipe id: " + swipe.id()
                               + ", " + swipe.state()
                               + ", position: " + swipe.position()
                               + ", direction: " + swipe.direction()
                               + ", speed: " + swipe.speed());
                    
                    process_swipe(frame.hands().count(), swipe.direction(), swipe.state());
                    
                    break;
                
                case TYPE_SCREEN_TAP:
                    ScreenTapGesture screenTap = new ScreenTapGesture(gesture);
                    System.out.println("Screen Tap id: " + screenTap.id()
                               + ", " + screenTap.state()
                               + ", position: " + screenTap.position()
                               + ", direction: " + screenTap.direction());
                    process_screen_tap(frame.hands().count(), frame.hands().leftmost().fingers().count());
                    break;
                
                case TYPE_KEY_TAP:
                    KeyTapGesture keyTap = new KeyTapGesture(gesture);
                    System.out.println("Key Tap id: " + keyTap.id()
                               + ", " + keyTap.state()
                               + ", position: " + keyTap.position()
                               + ", direction: " + keyTap.direction());
                    
                    process_key_tap(frame.hands().count(), frame.hands().leftmost().fingers().count());
                    //System.out.println("1 mano in scena: "+frame.hands().get(0)+" con "+frame.hands().get(0).fingers().count()+"dita!");
                    //System.out.println("2 mano in scena: "+frame.hands().get(1)+" con "+frame.hands().get(1).fingers().count()+"dita!");
                    //System.out.println("Mano sinistra: "+frame.hands().leftmost()+" con "+frame.hands().leftmost().fingers().count()+"dita!");
                    //System.out.println("Mano destra: "+frame.hands().rightmost()+" con "+frame.hands().rightmost().fingers().count()+"dita!");
                    break;
                    
                default:
                    System.out.println("Unknown gesture type.");
                    break;
            }
        }

        if (!frame.hands().isEmpty() || !gestures.isEmpty()) {
            /*System.out.println();*/
        }
    }
	
                public static double lap = 0;
                public static void process_circle(int n_mani, int sx_fingers, int dx_fingers, String direzione, double tempo){ // INTERPRETIAMO LA GESTURE DEL CERCHIO
                    try{
                        boolean vo = false;
                        if (tempo < 1){ vo = false; lap = 0;}
                        else if(tempo > lap+2){ vo = true; lap = tempo; } // Un aumento di volume/brightness ogni due secondi di gesture CIRCLE
                       
                            
                    if (n_mani == 2){ // Se nella scena ci sono due mani, si interagisce con le luci
                        if (vo)
                        if (direzione.equals("clockwise")){ // aumentiamo l'intensità
                            HUE.moreBrightness(sx_fingers); // increase brightness (input: light ID)
                        }else{ // diminuiamo l'intensità
                            HUE.lessBrightness(sx_fingers); // decrease brightness (input: light ID)
                        }
                    }else{ // altrimenti con il lettore multimediale
                        if (vo)
                        if (direzione.equals("clockwise")){ // aumentiamo il volume
                            web_vlc.more_vol();
                        }else{ // diminuiamo il volume
                            web_vlc.less_vol();
                        }
                    }
                    }catch(Exception e){
                        System.out.println(e);
                    }
                    
                }
                
             public static void process_screen_tap(int n_mani, int sx_fingers){
                 try{
                     if (n_mani == 2){
                         if (HUE.getEffectState(sx_fingers).equals("none"))
                            HUE.setLightEffectOn(sx_fingers);
                         else
                            HUE.setLightEffectOff(sx_fingers);
                     }
                 }catch (Exception e){
                     System.out.println(e);
                 }
             }
                
             public static void process_key_tap(int n_mani, int sx_fingers){
                    try{ // On tap event start playing music or turn off/on the lights.
                    
                    if (n_mani == 2){ // Turning on/off the lights
                       HUE.setLightState(sx_fingers, !HUE.getLightState(sx_fingers));
                    }
                    else // VLC interaction
                        if (play){
                            web_vlc.pause();
                            play = false;
                        }
                        else{
                            web_vlc.play();
                            play = true;
                            }
                    
                    
                    }catch (Exception e){
                        System.out.println(e);
                    }
             
             }
             
            public static void process_swipe(int n_mani, Vector direzione, State x){ // Next and previous track on VLC
                //System.out.println(direzione.getX());
                try{
               
                if (n_mani == 1)
                if(x.toString().equals("STATE_START"))
                if(direzione.getX() > 0)
                    web_vlc.next();
                else
                    web_vlc.prev();
                
                }catch (Exception e){
                   System.out.println(e);
                }
            }
            
}

class Sample {
    public static void main(String[] args){

        
        // Create a sample listener and controller
        SampleListener listener = new SampleListener();
        Controller controller = new Controller();

        // Have the sample listener receive events from the controller
        controller.addListener(listener);

        // Keep this process running until Enter is pressed
        System.out.println("Press Enter to quit...");
        try {
            System.in.read();
        } catch (IOException e) {
            e.printStackTrace();
        }

        // Remove the sample listener when done
        controller.removeListener(listener);
    }
}
